/*
** Equilibrium for two types, 2m and 2s, where 1 screens and
** both types pool on non-serious demands
*/

new;
#include ../gauss/calc-f.src;

/*
** construct the M partition of size n for the A matrix
*/
proc(1)=make_partition_m(n,delta,p);
    local m, left, right, lel, rel;

    lel = delta*(1-p);
    rel = delta*p;
    m = zeros(n,n);
    for i(1,n,1);
        left = i-1;
        right = i+1;
        if( left >= 1 ); m[i,left] = lel; endif;
        if( right <= n); m[i,right] = rel; endif;
    endfor;

    retp(m);
endp;


/*
** construct the matrix of coefficients A for N states of the world
*/
proc(1)=coeff_matrix(N,delta,p);
    local a,i,m;

    m = make_partition_m(N-1,delta,p);    @ make the M partition @
    i = eye(N-1);                        @ make the I partition @
    a = (i~m)|(m~i);                    @ concatenate them @

    retp(a);
endp;

/*
** construct the RHS vector Y for N states of the world
*/
proc(1)=rhs_matrix(N,delta,p,pie,b1,b2);
    local y, first, last, amid, bmid, dim;

    amid = {};
    bmid = {};
    first = pie-delta*(1-p)*pie-(1-delta)*b2;
    last  = pie-delta*p*pie-(1-delta)*b1;
    if(N > 2);
        dim = N-2;
        amid = ones(dim,1)*(pie-(1-delta)*b2);
        bmid = ones(dim,1)*(pie-(1-delta)*b1);
    endif;

    y = first|amid|bmid|last;
    retp(y);
endp;


// compute the optimal MPE offers from Y and the inverse of A
proc(2)=calc_MPE(N,delta,p,pie,b1,b2);
    local v,v1,v2,A,Y;

    A = coeff_matrix(N,delta,p);
    Y = rhs_matrix(N,delta,p,pie,b1,b2);
    v = inv(A)*Y;
    // add the terminal states 0 and N
    v1 = 0 | v[1:(N-1),.] | pie;
    v2 = pie | v[N:2*(N-1),.] | 0;
    retp(v1,v2);
endp;

// simple number formatting
proc(1)=std_val(n);
    local aStr;

    aStr=ftos(n,"%#*.*lf",3,2);
    retp(aStr);
endp;

// update belief from 1 battle; this takes some qS and 
// for each k calculates qS(k+1) and qS(k-1)
// assumes qS is valid and qM = 1-qS
proc(2)=update_belief(N,pL,pM,qS);
    local qSkp1,qSkm1;

    qSkp1 = zeros(N+1,1);
    qSkm1 = zeros(N+1,1);
    for k (2,N,1);
        qSkp1[k] = pL*qS/(pL*qS+pM*(1-qS));
        qSkm1[k] = (1-pL)*qS/((1-pL)*qS+(1-pM)*(1-qS));
    endfor;
    retp(qSkp1,qSkm1);
endp;

// maximum value of qS that can sustain separation in k
proc calc_maxQs(N,delta,pL,pM,pie,b1,b2,V2s);
    local z,zeta,maxQs;

    z = (delta*(pM-pL))/((1-delta)*(pie-b1-b2));
    maxQs = zeros(N+1,1);
    // k=0,..,N but indexed 1..N+1; calc for non-terminal k=1..N-1
    // which means indexing begins at 2 and goes to N
    for k (2,N,1);
        zeta = z*(V2s[k-1]-V2s[k+1]);
        maxQs[k] = zeta/(1+zeta);
    endfor;
    retp(maxQs);
endp;

// optimal offer that screens 2m from 2s
proc calc_x_screen(N,delta,pM,pie,b1,b2,V2s);
    local x;

    x = zeros(N+1,1);
    for k (2,N,1);
        x[k] = pie - (1-delta)*b2-delta*(pM*V2s[k+1]+(1-pM)*V2s[k-1]);
    endfor;
    x[N+1] = pie; // x[k=0]=0, x[k=N]=pie
    retp(x);
endp;

// 2m's expected payoff from pooling with 2s on non-serious demand
// this does not assume that 1 will screen in next period; instead,
// we calculate qS as expected to be updated from battle, and then
// use either screening payoff or, if qS is too large, 1's immediate
// concessionary payoff
proc calc_y_pool(N,delta,pM,pie,b2,x_screen,maxQs,qSkp1,qSkm1,V1s);
    local y,xp1,xm1;
    
    y = zeros(N+1,1);
    for k (2,N,1);
        if( qSkp1 <= maxQs);    // player 1 screens in k+1
            xp1 = x_screen[k+1];
        else;   // player 1 settles immediately in k+1 on V1s
            xp1 = V1s[k+1]; 
        endif;
        if( qSkm1 <= maxQs );   // player 1 screens in k-1
            xm1 = x_screen[k-1];
        else; // player 1 settles immediately in k-1 on V1s
            xm1 = V1s[k-1];
        endif;
        y[k] = (1-delta)*b2+delta*(pie-pM*xp1-(1-pM)*xm1);
    endfor;
    // y[k=N] = 0, y[k=0] = pie
    y[1] = pie;
    retp(y);
endp;

// this is an offer 1 would accept even though it reveals 2m's type
proc calc_y_accepted(N,delta,pM,pie,b1,V1m);
    local y;
    y = zeros(N+1,1);
    for k (2,N,1);
        y[k] = pie - (1-delta)*b1-delta*(pM*V1m[k+1]+(1-pM)*V1m[k-1]);
    endfor;
    y[1] = pie;
    retp(y);
endp;

// expected payoff to player 1 from screening with x(k)
proc calc_pay_screen(N,delta,pL,qS,pie,b1,x_screen,V2s);
    local pay;

    pay = zeros(N+1,1);
    for k (2,N,1);
        pay[k] = (1-qS)*x_screen[k]
            + qS*((1-delta)*b1+delta*(pie-pL*V2s[k+1]-(1-pL)*V2s[k-1]));
    endfor;
    pay[N+1] = pie;
    retp(pay);
endp;

// calculate x0(k) that separates 2w
proc calc_x_2w(N,delta,pH,pie,b2,xk);
    local z, x0;

    x0 = zeros(N+1,1);  
    for k (3,N-1,1);
        z = (pH^2)*xk[k+2]+2*pH*(1-pH)*xk[k]+((1-pH)^2)*xk[k-2];
        x0[k] = (1-delta^2)*(pie-b2)+(delta^2)*z;
    endfor;
    retp(x0);
endp;

proc calc_z(k,delta,pL,pie,b1,V2s);
    retp( (1-delta)*b1 + delta*(pie-pL*V2s[k+1]-(1-pL)*V2s[k-1]) );
endp;

// 1's payoff from fighting two battles & screening 2m from 2s
proc calc_wk(N,delta,pL,pM,pie,b1,b2,xk,V2s,qS1);
    local wk,ax,bx,cx;

    wk = zeros(N+1,1);
    for k (4,N-2,1);
        ax = (pL^2)*qS1*calc_z(k+2,delta,pL,pie,b1,V2s) + (pM^2)*(1-qS1)*xk[k+2];
        bx = 2*(pL*(1-pL)*qS1*calc_z(k,delta,pL,pie,b1,V2s)+pM*(1-pM)*(1-qS1)*xk[k]);
        cx = ((1-pL)^2)*qS1*calc_z(k-2,delta,pL,pie,b1,V2s)+((1-pM)^2)*(1-qS1)*xk[k-2];
        wk[k] = (1-delta^2)*b1+delta^2*(ax+bx+cx);
    endfor;
    retp(wk);
endp;

// expected payoff from screening 2w
proc calc_pay_2w(qW,x0,wk);
    retp( qW.*x0 + (1-qW).*wk );
endp;

// calculate deviation to screen 2w/2m from 2s
proc calc_pay_2m(N,delta,pL,pM,pie,b1,b2,V2s,qS);
    local pay,xd,z;

    pay = zeros(N+1,1);
    for k (4,N-2,1);
        z = calc_z(k,delta,pL,pie,b1,V2s);
        xd = pie-(1-delta)*b2-delta*(pM*V2s[k+1]+(1-pM)*V2s[k-1]);
        pay[k] = (1-qS)*xd + qS*z;
    endfor;
    retp(pay);
endp;

/*****************************************************************
** THE MAIN ROUTINE
******************************************************************/
//cls;

N = 10;           @ N >= 2 (number of contestable objects) @
// increase delta to increase maxQs
delta = .90;      @ common discount factor @
b1 = .10;         @ per-period payoff for player 1 @
b2 = .05;         @ per-period payoff for player 2 @
pie = 1;          @ size of the flow of benefits to divide @

k0 = 6;			@ starting state to test for @

// example in which starting in any k with qS=.1, player 1
// would screen in the next period regardless of battle outcome
// pL=.15, pM=.75, pie=1, b1=.1, b2=.05, delta=.95, N=10

// increase qS to increase qS updated from battle
qS = .1;		@ prior that 2 is strong @
qM = 1-qS;		@ prior that 2 is moderately strong @

// decrease pL (increase pM) to increase maxQs
pL = .15;		@ prob of winning against strong 2 @
pM = .75;		@ prob of winning against moderate 2 @

//pH = .7;		@ prob of winning against weak 2 @

k  = seqa(0,1,N+1); // all possible states, including terminal ones

{V1s,V2s} = calc_MPE(N,delta,pL,pie,b1,b2);
{V1m,V2m} = calc_MPE(N,delta,pM,pie,b1,b2);

maxQs = calc_maxQs(N,delta,pL,pM,pie,b1,b2,V2s);
{qSkp1,qSkm1} = update_belief(N,pL,pM,qS);

x_screen = calc_x_screen(N,delta,pM,pie,b1,b2,V2s);
y_pool = calc_y_pool(N,delta,pM,pie,b2,x_screen,maxQs,qSkp1,qSkm1,V1s);
y_deviate = calc_y_accepted(N,delta,pM,pie,b1,V1m);
p2m_deviate_profitable = y_deviate .> y_pool;
pay_screen = calc_pay_screen(N,delta,pL,qS,pie,b1,x_screen,V2s);
p1_screens = pay_screen .>= V1s;

f1s = calc_F1(N,pL,delta,b1,pie);
f2s = calc_F2(N,pL,delta,b2,pie);
f1m = calc_F1(N,pM,delta,b1,pie);
f2m = calc_F2(N,pM,delta,b1,pie);

format /mat /on /ma1 /rd 8,4;
print "   N     = " N;
print " delta   = " delta   "    qS = " qS;
print "   pie   = " pie     "    qM = " qM;
print "   b1    = " b1      "    pL = " pL;
print "   b2    = " b2      "    pM = " pM;
print "-------------------------------------------------------------------------------------------------------------";
print "    k      V1s(k)   V2s(k)   maxQs(k) qS(k+1)  qS(k-1)  x*(k)    pay1(x)  screen?   y*(k)   dev_y(k)  profit?";
print "-------------------------------------------------------------------------------------------------------------";
print k~V1s~V2s~maxQs~qSkp1~qSkm1~x_screen~pay_screen~p1_screens~y_pool~y_deviate~p2m_deviate_profitable;

// now do the three periods; since we want screening in t=2
// we want to make sure that qS is not too high after 2w
// separates by acceptance in t=0; that is, we want qS1
// that will work: qS < maxQsqM/(1-maxQs)

qS = 0.04;
qM = 0.36;
qW = 0.60;
qS1 = qS/(qS+qM);
pH = .95;

{V1s,V2s} = calc_MPE(N,delta,pL,pie,b1,b2);
{V1m,V2m} = calc_MPE(N,delta,pM,pie,b1,b2);

maxQs = calc_maxQs(N,delta,pL,pM,pie,b1,b2,V2s);

xk = calc_x_screen(N,delta,pM,pie,b1,b2,V2s);
x0k = calc_x_2w(N,delta,pH,pie,b2,xk);
wk = calc_wk(N,delta,pL,pM,pie,b1,b2,xk,V2s,qS1);
pay_eq = calc_pay_2w(qW,x0k,wk);
pay_dev = calc_pay_2m(N,delta,pL,pM,pie,b1,b2,V2s,qS);

zk = zeros(N+1,1);
for i (3,N-2,1);
    zk[i] = calc_z(i,delta,pL,pie,b1,V2s);
endfor;

print "   N     = " N       "    qW = " qW;
print " delta   = " delta   "    qS = " qS;
print "   pie   = " pie     "    qM = " qM;
print "   b1    = " b1      "    pL = " pL;
print "   b2    = " b2      "    pM = " pM;
print "   qS1   = " qS1     "    pH = " pH;
print "-------------------------------------------------------------------------------------------------------------";
print "    k       x0(k)   w(k)     U1(2w)   deviate   V1s(k)   V2s(k)   maxQs(k) qS(k+1)  qS(k-1)  x*(k)    pay1(x)  screen?   y*(k)   dev_y(k)  profit?";
print "-------------------------------------------------------------------------------------------------------------";
k~x0k~wk~pay_eq~pay_dev;
end;

